@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2010 Andy Spencer <andy753421@gmail.com>
+ * Copyright (C) 2009-2011 Andy Spencer <andy753421@gmail.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -128,10 +128,17 @@ G_MODULE_EXPORT void on_cleancache(GtkMenuItem *menu, AWeatherGui *self)
 	cleancache_r(cache);
 }
 
-G_MODULE_EXPORT void on_contents(GtkMenuItem *menu, AWeatherGui *self)
+G_MODULE_EXPORT void on_help(GtkMenuItem *menu, AWeatherGui *self)
 {
 	GError *err = NULL;
-	gchar *path = g_strdup(HTMLDIR "/aweather.html");
+	const gchar *name = gtk_buildable_get_name(GTK_BUILDABLE(menu));
+	gchar *page = g_str_has_suffix(name, "userguide") ? "userguide" :
+	              g_str_has_suffix(name, "manpage")   ? "aweather"  : NULL;
+	if (page == NULL) {
+		g_warning("Unknown help page: %s", page);
+		return;
+	}
+	gchar *path = g_strdup_printf("%s/%s.html", HTMLDIR, page);
 	g_strdelimit(path, "/", G_DIR_SEPARATOR);
 	gchar *argv[] = {"xdg-open", path, NULL};
 	g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &err);
@@ -143,6 +150,22 @@ G_MODULE_EXPORT void on_contents(GtkMenuItem *menu, AWeatherGui *self)
 	g_free(path);
 }
 
+void on_radar_changed(GtkMenuItem *menu, AWeatherGui *self)
+{
+	city_t *city = g_object_get_data(G_OBJECT(menu), "city");
+	grits_viewer_set_location(self->viewer,
+			city->pos.lat, city->pos.lon, EARTH_R/35);
+	/* Focus radar tab */
+	GtkWidget *config = aweather_gui_get_widget(self, "main_tabs");
+	gint       npages = gtk_notebook_get_n_pages(GTK_NOTEBOOK(config));
+	for (int i = 0; i < npages; i++) {
+		GtkWidget   *child  = gtk_notebook_get_nth_page(GTK_NOTEBOOK(config), i);
+		const gchar *plugin = gtk_notebook_get_tab_label_text(GTK_NOTEBOOK(config), child);
+		if (g_str_equal(plugin, "radar"))
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(config), i);
+	}
+}
+
 G_MODULE_EXPORT void on_quit(GtkMenuItem *menu, AWeatherGui *self)
 {
 	gtk_widget_destroy(GTK_WIDGET(self));
@@ -164,7 +187,9 @@ G_MODULE_EXPORT void on_fullscreen(GtkToggleAction *action, AWeatherGui *self)
 	GtkWidget *toolbar = aweather_gui_get_widget(self, "main_toolbar");
 	GtkWidget *sidebar = aweather_gui_get_widget(self, "main_sidebar");
 	GtkWidget *tabs    = aweather_gui_get_widget(self, "main_tabs");
+	static gboolean menushow = TRUE; // Toolbar can be always disabled
 	if (gtk_toggle_action_get_active(action)) {
+		menushow = gtk_widget_get_visible(menu);
 		gtk_window_fullscreen(GTK_WINDOW(self));
 		gtk_widget_hide(menu);
 		_gtk_widget_autohide(toolbar, GTK_WIDGET(self->viewer), GTK_POS_TOP);
@@ -172,7 +197,7 @@ G_MODULE_EXPORT void on_fullscreen(GtkToggleAction *action, AWeatherGui *self)
 		_gtk_widget_autohide(tabs,    GTK_WIDGET(self->viewer), GTK_POS_BOTTOM);
 	} else {
 		gtk_window_unfullscreen(GTK_WINDOW(self));
-		gtk_widget_show(menu);
+		gtk_widget_set_visible(menu, menushow);
 		_gtk_widget_autoshow(toolbar);
 		_gtk_widget_autoshow(sidebar);
 		_gtk_widget_autoshow(tabs);
@@ -242,13 +267,15 @@ G_MODULE_EXPORT void on_time_changed(AWeatherGui *self)
 			(guint*)&tm.tm_mon, (guint*)&tm.tm_mday);
 	tm.tm_year -= 1900;
 
-	GtkTreeIter   iter;
-	GtkTreePath  *path  = NULL;
-	GtkWidget    *view  = aweather_gui_get_widget(self, "main_time");
-	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
+	GtkTreePath *path = NULL;
+	GtkWidget   *view = aweather_gui_get_widget(self, "main_time");
 	gtk_tree_view_get_cursor(GTK_TREE_VIEW(view), &path, NULL);
-	gtk_tree_model_get_iter(model, &iter, path);
-	gtk_tree_model_get(model, &iter, 1, &tm.tm_hour, 2, &tm.tm_min, -1);
+	if (path) {
+		GtkTreeIter   iter;
+		GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
+		gtk_tree_model_get_iter(model, &iter, path);
+		gtk_tree_model_get(model, &iter, 1, &tm.tm_hour, 2, &tm.tm_min, -1);
+	}
 
 	grits_viewer_set_time(self->viewer, mktime(&tm));
 }
@@ -290,6 +317,14 @@ static void update_time_widget(GritsViewer *viewer, time_t time, AWeatherGui *se
 	//}
 }
 
+G_MODULE_EXPORT gboolean on_configure(AWeatherGui *self, GdkEventConfigure *config)
+{
+	g_debug("AWeatherGui: on_configure - %dx%d", config->width, config->height);
+	grits_prefs_set_integer(self->prefs, "aweather/width",  config->width);
+	grits_prefs_set_integer(self->prefs, "aweather/height", config->height);
+	return FALSE;
+}
+
 /* Prefs callbacks */
 G_MODULE_EXPORT void on_offline(GtkToggleAction *action, AWeatherGui *self)
 {
@@ -355,7 +390,7 @@ static void site_setup(AWeatherGui *self)
 			gtk_tree_store_append(store, &state, NULL);
 			gtk_tree_store_set   (store, &state, 0, cities[i].code,
 					                     1, cities[i].name, -1);
-		} else {
+		} else if (cities[i].type == LOCATION_CITY) {
 			gtk_tree_store_append(store, &city, &state);
 			gtk_tree_store_set   (store, &city, 0, cities[i].code,
 				                            1, cities[i].name, -1);
@@ -368,6 +403,29 @@ static void site_setup(AWeatherGui *self)
 			GTK_CELL_RENDERER(renderer), combo_sensitive, NULL, NULL);
 }
 
+static void menu_setup(AWeatherGui *self)
+{
+	GtkWidget *menu = aweather_gui_get_widget(self, "main_menu_radar");
+	GtkWidget *states=NULL, *state=NULL, *sites=NULL, *site=NULL;
+
+	states = gtk_menu_new();
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu), states);
+
+	for (int i = 0; cities[i].type; i++) {
+		if (cities[i].type == LOCATION_STATE) {
+			state = gtk_menu_item_new_with_label(cities[i].name);
+			sites = gtk_menu_new();
+			gtk_menu_shell_append(GTK_MENU_SHELL(states), state);
+			gtk_menu_item_set_submenu(GTK_MENU_ITEM(state), sites);
+		} else if (cities[i].type == LOCATION_CITY) {
+			site  = gtk_menu_item_new_with_label(cities[i].name);
+			gtk_menu_shell_append(GTK_MENU_SHELL(sites), site);
+			g_object_set_data(G_OBJECT(site), "city", &cities[i]);
+			g_signal_connect(site, "activate", G_CALLBACK(on_radar_changed), self);
+		}
+	}
+}
+
 static void prefs_setup(AWeatherGui *self)
 {
 	/* Set values */
@@ -387,6 +445,7 @@ static void prefs_setup(AWeatherGui *self)
 		GtkTreeIter iter;
 		if (gtk_tree_model_find_string(model, &iter, NULL, 0, is))
 			gtk_combo_box_set_active_iter(GTK_COMBO_BOX(isw), &iter);
+		g_free(is);
 	}
 
 	/* Load Plugins */
@@ -561,10 +620,17 @@ static void aweather_gui_parser_finished(GtkBuildable *_self, GtkBuilder *builde
 
 	/* Misc, helpers */
 	site_setup(self);
+	menu_setup(self);
 	time_setup(self);
 	prefs_setup(self);
 	icons_setup(self);
 
+	/* Default size */
+	gint width  = grits_prefs_get_integer(self->prefs, "aweather/width",  NULL);
+	gint height = grits_prefs_get_integer(self->prefs, "aweather/height", NULL);
+	if (width && height)
+		gtk_window_set_default_size(GTK_WINDOW(self), width, height);
+
 	/* Connect signals */
 	gtk_builder_connect_signals(self->builder, self);
 	g_signal_connect(self, "key-press-event",
@@ -601,16 +667,19 @@ static void aweather_gui_dispose(GObject *_self)
 	g_debug("AWeatherGui: dispose");
 	AWeatherGui *self = AWEATHER_GUI(_self);
 	if (self->plugins) {
-		grits_plugins_free(self->plugins);
+		GritsPlugins *plugins = self->plugins;
 		self->plugins = NULL;
+		grits_plugins_free(plugins);
 	}
 	if (self->builder) {
-		g_object_unref(self->builder);
+		GtkBuilder *builder = self->builder;
 		self->builder = NULL;
+		g_object_unref(builder);
 	}
 	if (self->prefs) {
-		g_object_unref(self->prefs);
+		GritsPrefs *prefs = self->prefs;
 		self->prefs = NULL;
+		g_object_unref(prefs);
 	}
 	G_OBJECT_CLASS(aweather_gui_parent_class)->dispose(_self);
 }
