@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2010 Andy Spencer <andy753421@gmail.com>
+ * Copyright (C) 2009-2011 Andy Spencer <andy753421@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,10 @@
 #include <gtk/gtk.h>
 #include <glib/gstdio.h>
 
+#ifdef MAC_INTEGRATION
+#include <gtkosxapplication.h>
+#endif
+
 #include <grits.h>
 
 #include "aweather-gui.h"
@@ -47,6 +51,12 @@ static void log_func(const gchar *log_domain, GLogLevelFlags log_level,
 	}
 }
 
+static void xdg_open(GtkWidget *widget, const gchar *link, gpointer user_data)
+{
+	gchar *argv[] = {"xdg-open", (gchar*)link, NULL};
+	g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL);
+}
+
 static void on_log_level_changed(GtkSpinButton *spinner, AWeatherGui *self)
 {
 	g_message("main: log_level_changed");
@@ -58,19 +68,23 @@ static void set_location_time(AWeatherGui *gui, char *site, char *time)
 	/* Set time
 	 *   Do this before setting setting location
 	 *   so that it doesn't refresh twice */
-	struct tm tm = {};
-	strptime(time, "%Y-%m-%d %H:%M", &tm);
-	time_t sec = mktime(&tm);
-	if (sec > 0)
-		grits_viewer_set_time(gui->viewer, sec);
-	g_debug("date = [%s] == %lu\n", time, sec);
+	if (time) {
+		int year, mon, day, hour, min;
+		sscanf(time, "%d-%d-%d %d:%d", &year, &mon, &day, &hour, &min);
+		time_t sec = mktime(&(struct tm){0, year-1900, mon-1, day, hour, min});
+		if (sec > 0)
+			grits_viewer_set_time(gui->viewer, sec);
+		g_debug("date = [%s] == %lu\n", time, sec);
+	}
 
 	/* Set location */
-	for (city_t *city = cities; city->type; city++) {
-		if (city->type == LOCATION_CITY && g_str_equal(city->code, site)) {
-			grits_viewer_set_location(gui->viewer,
-				city->pos.lat, city->pos.lon, EARTH_R/35);
-			break;
+	if (site) {
+		for (city_t *city = cities; city->type; city++) {
+			if (city->type == LOCATION_CITY && g_str_equal(city->code, site)) {
+				grits_viewer_set_location(gui->viewer,
+					city->pos.lat, city->pos.lon, EARTH_R/35);
+				break;
+			}
 		}
 	}
 }
@@ -81,6 +95,19 @@ static void set_toggle_action(AWeatherGui *gui, const char *action, gboolean ena
 	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(object), enabled);
 }
 
+static void setup_mac(AWeatherGui *gui)
+{
+#ifdef MAC_INTEGRATION
+	GtkWidget *menu = aweather_gui_get_widget(gui, "main_menu");
+	GtkOSXApplication *app = g_object_new(GTK_TYPE_OSX_APPLICATION, NULL);
+	gtk_widget_hide(menu);
+	gtk_osxapplication_set_menu_bar(app, GTK_MENU_SHELL(menu));
+	gtk_osxapplication_set_use_quartz_accelerators(app, TRUE);
+	gtk_osxapplication_ready(app);
+	//gtk_osxapplication_sync_menubar(app)
+#endif
+}
+
 /********
  * Main *
  ********/
@@ -88,8 +115,8 @@ int main(int argc, char *argv[])
 {
 	/* Defaults */
 	gint     debug      = 2; // G_LOG_LEVEL_WARNING
-	gchar   *site       = "";
-	gchar   *time       = "";
+	gchar   *site       = NULL;
+	gchar   *time       = NULL;
 	gboolean autoupdate = FALSE;
 	gboolean offline    = FALSE;
 	gboolean fullscreen = FALSE;
@@ -112,6 +139,9 @@ int main(int argc, char *argv[])
 		{NULL}
 	};
 
+	/* All times in UTC */
+	g_setenv("TZ", "UTC", TRUE);
+
 	/* Init */
 	GError *error = NULL;
 	g_thread_init(NULL);
@@ -122,6 +152,11 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 
+	/* Use external handler for link buttons */
+	gtk_link_button_set_uri_hook((GtkLinkButtonUriFunc)xdg_open, NULL, NULL);
+	gtk_about_dialog_set_url_hook((GtkAboutDialogActivateLinkFunc)xdg_open, NULL, NULL);
+	gtk_about_dialog_set_email_hook((GtkAboutDialogActivateLinkFunc)xdg_open, NULL, NULL);
+
 	/* Setup debug level for aweather_gui_new */
 	g_log_set_handler(NULL, G_LOG_LEVEL_MASK, log_func, NULL);
 	log_levels = int2log(opt_debug >= 0 ? opt_debug : debug);
@@ -159,10 +194,12 @@ int main(int argc, char *argv[])
 	grits_viewer_set_offline(gui->viewer, offline);
 	set_toggle_action(gui, "update",     autoupdate);
 	set_toggle_action(gui, "fullscreen", fullscreen);
+	g_free(prefs_site);
 
 	/* Done with init, show gui */
 	gtk_widget_show_all(GTK_WIDGET(gui));
 	set_toggle_action(gui, "fullscreen", fullscreen); // Resest widget hiding
+	setup_mac(gui);	// done after show_all
 	gtk_main();
 	gdk_threads_leave();
 	gdk_display_close(gdk_display_get_default());
