@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2010 Andy Spencer <andy753421@gmail.com>
+ * Copyright (C) 2009-2011 Andy Spencer <andy753421@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 
 #include <config.h>
 #include <math.h>
-#include <GL/gl.h>
 #include <glib/gstdio.h>
 #include <grits.h>
 #include <rsl.h>
@@ -211,7 +210,7 @@ void aweather_level2_draw(GritsObject *_level2, GritsOpenGL *opengl)
 
 	/* Draw wsr88d */
 	Sweep *sweep = level2->sweep;
-	glDisable(GL_ALPHA_TEST);
+	//glDisable(GL_ALPHA_TEST);
 	glDisable(GL_CULL_FACE);
 	glDisable(GL_LIGHTING);
 	glEnable(GL_TEXTURE_2D);
@@ -324,7 +323,7 @@ void aweather_level2_set_iso(AWeatherLevel2 *level2, gfloat level)
 		vol->disp = GRITS_VOLUME_SURFACE;
 		GRITS_OBJECT(vol)->center = GRITS_OBJECT(level2)->center;
 		grits_viewer_add(GRITS_OBJECT(level2)->viewer,
-				GRITS_OBJECT(vol), GRITS_LEVEL_WORLD, TRUE);
+				GRITS_OBJECT(vol), GRITS_LEVEL_WORLD+5, TRUE);
 		level2->volume = vol;
 	}
 	if (ISO_MIN < level && level < ISO_MAX) {
@@ -379,9 +378,9 @@ AWeatherLevel2 *aweather_level2_new_from_file(const gchar *file, const gchar *si
 
 	/* Load the radar file */
 	RSL_read_these_sweeps("all", NULL);
-	g_message("read start");
+	g_debug("AWeatherLevel2: rsl read start");
 	Radar *radar = RSL_wsr88d_to_radar(raw, (gchar*)site);
-	g_message("read done");
+	g_debug("AWeatherLevel2: rsl read done");
 	g_free(raw);
 	if (!radar)
 		return NULL;
@@ -393,8 +392,8 @@ static void _on_sweep_clicked(GtkRadioButton *button, gpointer _level2)
 {
 	AWeatherLevel2 *level2 = _level2;
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) {
-		gint type = (gint)g_object_get_data(G_OBJECT(button), "type");
-		gint elev = (gint)g_object_get_data(G_OBJECT(button), "elev");
+		gint type = (glong)g_object_get_data(G_OBJECT(button), "type");
+		gint elev = (glong)g_object_get_data(G_OBJECT(button), "elev");
 		aweather_level2_set_sweep(level2, type, (float)elev/100);
 		//level2->colormap = level2->sweep_colors;
 	}
@@ -420,7 +419,7 @@ GtkWidget *aweather_level2_get_config(AWeatherLevel2 *level2)
 
 	/* Add date */
 	gchar *date_str = g_strdup_printf("<b><i>%04d-%02d-%02d %02d:%02d</i></b>",
-			radar->h.year, radar->h.month, radar->h.day,
+			radar->h.year, radar->h.month, radar->h.day+1,
 			radar->h.hour, radar->h.minute);
 	GtkWidget *date_label = gtk_label_new(date_str);
 	gtk_label_set_use_markup(GTK_LABEL(date_label), TRUE);
@@ -476,9 +475,9 @@ GtkWidget *aweather_level2_get_config(AWeatherLevel2 *level2)
 			g_object_set(button, "draw-indicator", FALSE, NULL);
 			gtk_box_pack_end(GTK_BOX(elev_box), button, TRUE, TRUE, 0);
 
-			g_object_set_data(G_OBJECT(button), "level2", (gpointer)level2);
-			g_object_set_data(G_OBJECT(button), "type",   (gpointer)vi);
-			g_object_set_data(G_OBJECT(button), "elev",   (gpointer)(int)(elev*100));
+			g_object_set_data(G_OBJECT(button), "level2", level2);
+			g_object_set_data(G_OBJECT(button), "type", (gpointer)(guintptr)vi);
+			g_object_set_data(G_OBJECT(button), "elev", (gpointer)(guintptr)(elev*100));
 			g_signal_connect(button, "clicked", G_CALLBACK(_on_sweep_clicked), level2);
 		}
 	}
@@ -516,7 +515,8 @@ static void aweather_level2_dispose(GObject *_level2)
 	AWeatherLevel2 *level2 = AWEATHER_LEVEL2(_level2);
 	g_debug("AWeatherLevel2: dispose - %p", _level2);
 	if (level2->volume) {
-		grits_viewer_remove(GRITS_OBJECT(level2)->viewer, level2->volume);
+		grits_viewer_remove(GRITS_OBJECT(level2->volume)->viewer,
+				GRITS_OBJECT(level2->volume));
 		level2->volume = NULL;
 	}
 	G_OBJECT_CLASS(aweather_level2_parent_class)->dispose(_level2);
